#ifndef __SCENEINTMD_H__
#define __SCENEINTMD_H__

#include "exttypes.h"
#include <geommath/geommath.h>
#include "surfaceintmd.h"
#include "textureintmd.h"
#include "keylistintmd.h"


// intermediate representations of the various scene components


class CTransformation
{
public:
	int32 GetNumPosKeys() const { return m_vPosArray.GetNumKeys(); }
	int32 GetNumRotKeys() const { return m_qRotArray.GetNumKeys(); }
	int32 GetNumScaleKeys() const { return m_fUniScaleArray.GetNumKeys(); }
	Vec3 GetPosKey(const int32 index) const { return m_vPosArray.GetKey(index); }
	Quat GetRotKey(const int32 index) const { return m_qRotArray.GetKey(index); }
	float GetScaleKey(const int32 index) const { return m_fUniScaleArray.GetKey(index); }
	Vec3 GetPosValue(const float t) const { return m_vPosArray.GetValue(t); }
	Quat GetRotValue(const float t) const { return m_qRotArray.GetValue(t); }
	float GetScaleValue(const float t) const { return m_fUniScaleArray.GetValue(t); }

protected:
	CKey3ListIntmd m_vPosArray;
	CQuatListIntmd m_qRotArray;
	CKey1ListIntmd m_fUniScaleArray;
};


class CMeshIntmd
{
	friend class CSceneCreatorIntmd;
	friend class CSceneIntmd;

public:
	char * GetName() const { return m_pMeshName; }
	int32 GetNumSurfaces() const { return m_iNumSurfs; }
	CSurfaceIntmd& GetSurface(int32 iNumber) const { return m_pSurfaces[iNumber]; }
		
	CMeshIntmd()
	{
		m_iNumSurfs = 0;
		m_pSurfaces = NULL;
		m_pMeshName = NULL;
	}

private:
	void IncSurfBuffer(const int32 numverts, const int32 numstrips, const char matref[]);
	void SetName(const char name[]);
	bool operator !=(const CMeshIntmd &mesh);

	int32 m_iNumSurfs;
	CSurfaceIntmd * m_pSurfaces;

	char * m_pMeshName;

private:	// internal
	void CleanUp();
};

class CSceneIntmd;

class CMeshInstanceIntmd : public CTransformation
{
	friend class CSceneCreatorIntmd;
	friend class CSceneIntmd;

public:
	// you can get the mesh from CSceneIntmd
	// calling GetMesh() with this index as arg
	int GetMeshIndex() const;
	char * GetName() const { return m_pMeshInstName; }
	
	
	CMeshInstanceIntmd()
	{
		m_pMeshInstName = NULL;
		m_pOwnerScene = NULL;
		m_pMeshRef = NULL;
	}

private:
	void SetName(const char name[]);
	void SetMeshref(const char meshref[]);
	
	char * m_pMeshInstName;
	
private:	// internal
	void CleanUp();
	CSceneIntmd * m_pOwnerScene;
	char * m_pMeshRef;
};

class CLightIntmd : public CTransformation
{
	friend class CSceneCreatorIntmd;
	friend class CSceneIntmd;

public:
	// to be used for any light type
	int32 GetNumColorKeys() const { return m_vColorArray.GetNumKeys(); }
	int32 GetNumNearKeys() const { return m_fNearArray.GetNumKeys(); }
	int32 GetNumFarKeys() const { return m_fFarArray.GetNumKeys(); }
	Vec3 GetColorKey(const int32 index) const { return m_vColorArray.GetKey(index); }
	float GetNearKey(const int32 index) const { return m_fNearArray.GetKey(index); }
	float GetFarKey(const int32 index) const { return m_fFarArray.GetKey(index); }
	Vec3 GetColorValue(const float t) const { return m_vColorArray.GetValue(t); }
	float GetNearValue(const float t) const { return m_fNearArray.GetValue(t); }
	float GetFarValue(const float t) const { return m_fFarArray.GetValue(t); }

	bool IsSpot() const { return m_bIsSpot; }
	char * GetName() const { return m_pLgtName; }


	// to be used only if IsSpot() returns true
	int32 GetNumHotSpotKeys() const { return m_fHotSpotArray.GetNumKeys(); }
	int32 GetNumFallOffKeys() const { return m_fFallOffArray.GetNumKeys(); }
	int32 GetNumAspectKeys() const { return m_fAspectArray.GetNumKeys(); }	
	float GetHotSpotKey(const int32 index) const { return m_fHotSpotArray.GetKey(index); }
	float GetFallOffKey(const int32 index) const { return m_fFallOffArray.GetKey(index); }
	float GetAspectKey(const int32 index) const { return m_fAspectArray.GetKey(index); }
	float GetHotSpotValue(const float t) const { return m_fHotSpotArray.GetValue(t); }
	float GetFallOffValue(const float t) const { return m_fFallOffArray.GetValue(t); }
	float GetAspectValue(const float t) const { return m_fAspectArray.GetValue(t); }

	bool IsRectangular() const { return m_bIsRectangular; }

	CLightIntmd() : m_bIsSpot(false), m_bIsRectangular(false), m_pLgtName(NULL) {}
	
private:
	CKey3ListIntmd m_vColorArray;
	CKey1ListIntmd m_fNearArray;
	CKey1ListIntmd m_fFarArray;

	// these are only effective if m_bIsSpot is true
	CKey1ListIntmd m_fHotSpotArray;
	CKey1ListIntmd m_fFallOffArray;
	CKey1ListIntmd m_fAspectArray;
	bool m_bIsSpot;
	bool m_bIsRectangular;

	void SetName(const char name[]);
	
	char * m_pLgtName;

private:	// internal
	void CleanUp();
};

class CCameraIntmd : public CTransformation
{
	friend class CSceneCreatorIntmd;
	friend class CSceneIntmd;
	
public:
	int32 GetNumFovKeys() const { return m_fFovArray.GetNumKeys(); }
	float GetFovKey(const int32 index) const { return m_fFovArray.GetKey(index); }
	float GetFovValue(const float t) const { return m_fFovArray.GetValue(t); }
	float GetAspect() const { return m_fAspect; }
	char * GetName() const { return m_pCamName; }
	
	CCameraIntmd() : m_fAspect(4.0f/3.0f), m_pCamName(NULL) {}
	
private:
	CKey1ListIntmd m_fFovArray;	// Field Of View in radians (fov < 180 degrees)
	float m_fAspect;	// default value is 4:3

	void SetName(const char name[]);
	
	char * m_pCamName;

private:	// internal
	void CleanUp();
};

class CSceneIntmd
{
	friend class CSceneCreatorIntmd;

public:
	int32 GetNumMaterials()	const { return m_iNumMats; }
	int32 GetNumTextures() const { return m_iNumTextures; }
	int32 GetNumMeshes() const { return m_iNumMeshes; }
	int32 GetNumMeshInstances() const { return m_iNumMeshInstances; }
	int32 GetNumLights() const { return m_iNumLights; }
	int32 GetNumCameras() const { return m_iNumCameras; }
	int32 GetNumNullMeshes() const { return m_iNumNullMeshes; }
	int32 GetNumNullMeshInstances() const { return m_iNumNullMeshInstances; }

	CMaterialIntmd& GetMaterial(int iNumber) const { return m_pMaterials[iNumber]; }
	CMeshIntmd& GetMesh(int iNumber) const { return m_pMeshes[iNumber]; }
	CMeshInstanceIntmd& GetMeshInstance(int iNumber) const { return m_pMeshInstances[iNumber]; }
	CLightIntmd& GetLight(int iNumber) const { return m_pLights[iNumber]; }
	CCameraIntmd& GetCamera(int iNumber) const { return m_pCameras[iNumber]; }
	CTextureIntmd& GetTexture(int iNumber) const { return m_pTextures[iNumber]; }
		

	CSceneIntmd()
	{
		m_pMeshes = NULL; m_iNumMeshes = 0;
		m_pMeshInstances = NULL; m_iNumMeshInstances = 0;
		m_pLights = NULL; m_iNumLights = 0;
		m_pCameras = NULL; m_iNumCameras = 0;
		m_pMaterials = NULL; m_iNumMats = 0;
		m_pTextures = NULL; m_iNumTextures = 0;
	}
	
	
protected:
	int32 m_iNumMeshes;
	int32 m_iNumMeshInstances;
	int32 m_iNumLights;
	int32 m_iNumCameras;

	// nulls are located at the end of m_pMeshes and m_pMeshInstances
	// number of non-null meshes is m_iNumMeshes-m_iNumNullMeshes
	// and same principle goes for meshinstances.
	// non-nulls are ofcourse at the beginning of the buffers.
	int32 m_iNumNullMeshes;
	int32 m_iNumNullMeshInstances;
	//int32 m_iPad1, m_iPad2;
	
	CMeshIntmd * m_pMeshes;
	CMeshInstanceIntmd * m_pMeshInstances;
	CLightIntmd * m_pLights;
	CCameraIntmd * m_pCameras;

protected:
	void IncMaterialBuffer(const char mat_name[]);
	void delMaterial(const int32 num);
	void IncMeshBuffer(const char mesh_name[]);
	void delMesh(const int32 num);
	void IncMeshInstanceBuffer(const char meshinst_name[]);
	void IncCameraBuffer(const char cam_name[]);
	void IncLightBuffer(const char lgt_name[]);
	void SwapMeshes(const int32 m1, const int32 m2);
	void SwapMeshInstances(const int32 m1, const int32 m2);
	CTextureIntmd * InsertIntoTexList( CTextureIntmd * pTexEntry, bool * pFoundMatch );

	
	int32 m_iNumMats;
	CMaterialIntmd * m_pMaterials;

	int32 m_iNumTextures;
	CTextureIntmd * m_pTextures;

private:	// cleanup team
	void CleanUp();
	void CleanUpMeshes();
	void CleanUpTextures();
};

#endif
