#include "intmd/surfaceintmd.h"
#include "intmd/textureintmd.h"
#include "intmd/sceneintmd.h"
#include "p2lassert.h"

bool CMaterialIntmd::operator !=(const CMaterialIntmd &mat)
{
	bool bRes = true;

	// compare all data
	if(strcmp(GetName(), mat.GetName()) == 0)
	{
		const CTextureSeqIntmd &diff1 = m_cDiffTexture;
		const CTextureSeqIntmd &diff2 = mat.m_cDiffTexture;
		const CTextureSeqIntmd &refl1 = m_cReflTexture;
		const CTextureSeqIntmd &refl2 = mat.m_cReflTexture;
		const CTextureSeqIntmd &bump1 = m_cBumpTexture;
		const CTextureSeqIntmd &bump2 = mat.m_cBumpTexture;
		const CTextureSeqIntmd &spec1 = m_cSpecTexture;
		const CTextureSeqIntmd &spec2 = mat.m_cSpecTexture;

		const bool bTst1 = !(diff1!=diff2);
		const bool bTst2 = !(refl1!=refl2);
		const bool bTst3 = !(bump1!=bump2);
		const bool bTst4 = !(spec1!=spec2);
		
		bRes = bTst1 && bTst2 && bTst3 && bTst4;
	}
	
	return bRes;
}

void CMaterialIntmd::SetName(const char name[])
{
	assert(m_pMatName == NULL);
	int32 len = strlen(name) + 1;

	if((m_pMatName=new char[len]) == NULL)
	{
		printf("allocation error\n");
		assert(false);
	}

	strcpy(m_pMatName, name);
}


bool CSurfaceIntmd::operator !=(const CSurfaceIntmd &surf)
{
	bool bRes = true;
	
	// compare all data
	if(strcmp(m_pMaterialRef, surf.m_pMaterialRef) == 0)
	{
		if(GetArrayLen()==surf.GetArrayLen() && GetNumStrips()==surf.GetNumStrips())
		{
			if(memcmp(m_pVerts, surf.m_pVerts, sizeof(Vec3)*GetArrayLen())==0)
			{
				if(memcmp(m_pNormals, surf.m_pNormals, sizeof(Vec3)*GetArrayLen())==0)
				{
					if(memcmp(m_pSTs, surf.m_pSTs, sizeof(Vec3)*GetArrayLen())==0)
					{
						for(int s=0; s<GetNumStrips(); s++)
						{
							CStripIntmd &strip = GetStrip(s);

							int k=0;
							bool bFoundStrip=false;
							while(k<GetNumStrips() && !bFoundStrip)
								bFoundStrip = !(strip != surf.GetStrip(k++));

							bRes = bRes && bFoundStrip;
						}
					}
				}
			}
		}
	}
	
	return bRes;
}

int32 CSurfaceIntmd::GetMaterialIndex() const
{
	int32 index=-1;
	for(int n=0; n<m_pOwnerScene->GetNumMaterials(); n++)
	{
		if(strcmp(m_pMaterialRef, m_pOwnerScene->GetMaterial(n).GetName())==0)
		{
			assert(index == -1);	// no duplicates
			index = n;
		}
	}
	assert(index != -1);
	
	return index;
}


void CSurfaceIntmd::Setup(const int32 numverts, const int32 numstrips, const char matref[])
{
	m_iNumStrips = numstrips;
	
	m_pMaterialRef = new char[strlen(matref)+1];
	assert(m_pMaterialRef != NULL);
	strcpy(m_pMaterialRef, matref);

	m_iNumStrips = numstrips;
	m_pStrips = new CStripIntmd[numstrips];
	assert(m_pStrips != NULL);

	m_iLenArrays = numverts;
	m_pVerts = new Vec3[m_iLenArrays];
	assert(m_pVerts != NULL);
	m_pNormals = new Vec3[m_iLenArrays];
	assert(m_pNormals != NULL);
	m_pTangents = new Vec3[m_iLenArrays];
	assert(m_pTangents != NULL);
	m_pBiNormals = new Vec3[m_iLenArrays];
	assert(m_pBiNormals != NULL);
	m_pSTs = new Vec3[m_iLenArrays];
	assert(m_pSTs != NULL);
}


bool CStripIntmd::operator !=(const CStripIntmd &strip)
{
	bool bRes = true;

	// compare all data
	if(GetArrayLen()==strip.GetArrayLen())
		if(memcmp(m_pIndices, strip.m_pIndices, sizeof(int32)*GetArrayLen())==0)
			bRes = false;
	
	return bRes;
}

void CStripIntmd::Setup(const int32 len)
{
	m_pIndices = new int32[len];
	assert(m_pIndices!=NULL);
	m_len = len;
}


// clean up
void CMaterialIntmd::CleanUp()
{
	m_cDiffTexture.CleanUp();
	m_cReflTexture.CleanUp();
	m_cBumpTexture.CleanUp();
	m_cSpecTexture.CleanUp();
	if(m_pMatName!=NULL) { delete [] m_pMatName; m_pMatName = NULL; }
}

void CSurfaceIntmd::CleanUp()
{
	// this is just a ref, material's already been deleted
	if(m_pMaterialRef!=NULL) { delete [] m_pMaterialRef; m_pMaterialRef=NULL; }
	
	// erase strip indices
	for(int s=0; s<m_iNumStrips; s++)
		m_pStrips[s].CleanUp();
	if(m_pStrips!=NULL) { delete [] m_pStrips; m_pStrips=NULL; }
	m_iNumStrips=0;

	// erase remaining data
	m_iLenArrays=0;
	if(m_pVerts!=NULL) { delete [] m_pVerts; m_pVerts=NULL; }
	if(m_pNormals!=NULL) { delete [] m_pNormals; m_pNormals=NULL; }
	if(m_pTangents!=NULL) { delete [] m_pTangents; m_pTangents=NULL; }
	if(m_pBiNormals!=NULL) { delete [] m_pBiNormals; m_pBiNormals=NULL; }
	if(m_pSTs!=NULL) { delete [] m_pSTs; m_pSTs=NULL; }
}

void CStripIntmd::CleanUp()
{
	m_len=0;
	if(m_pIndices!=NULL) { delete [] m_pIndices; m_pIndices=NULL; }
}
